package gpsys.cos;

/**
 * The terrain consists of a torroidal grid of squares upon which the "grass
 * grows".  The terrain can be grown, and cut.  In addition, an observer can be
 * set which gets update messages each time the state of the terrain changes.
 *
 * @see	gpsys.cos.TerrainObserver
 *
 * @version             1.1, 24 Mars 1999
 * @author      <a href="mailto:epierre@e-nef.com">Emmanuel PIERRE</a>
 *                      <address>DEA IARFA,</address>
 */
public class Terrain implements java.io.Serializable {
	/**
	 * Used to hold the state of each square of the terrain.
	 */
	int[][] terrain;
	int l0 = 3*4;
	int l1 = 3*4;
	int l2 = 3*4;
	int ttl = 0;  	// time lived by the terrain
	int csnrj = lmax*3;  // available energy at the CS

	public static final int lmax = 30*4; 
	public static final int L0   = 6;
	public static final int L1   = 5;
	public static final int L2   = 4;
	public static final int R1   = 3;
	public static final int R2   = 2;
	public static final int CS   = 1;

	/**
	 * True if the observer is wants to received state change information.
	 */
	transient boolean		observed;
	/**
	 * A reference to an observer interested in monitoring the state of this
	 * terrain.
	 */
	transient TerrainObserver	observer;

	/**
	 * Create a square terrain of the specified dimension, with no observer.
	 *
	 * @param dimension	The value to be used for both the height and width
	 *					of the terrain.
	 */
	public Terrain(int dimension) {
		terrain = new int[dimension][dimension];
		observer = null;
		observed = false;
	}

	/**
	 * Set the observer of the terrain to the specified observer.
	 *
	 * @param lo	The observer of the Terrain.
	 */
	public void setObserver(TerrainObserver lo) {
		observer = lo;
	}

	/**
	 * Sets the all of the squares on the terrain to initial state.  The
	 * observer is notified of this change.
	 */
	public final void reinit() {
		for (int i = 0; i < terrain.length; i++)
			for (int j = 0; j < terrain.length; j++)
				terrain[i][j] = 0;
		//terrain[8][1]=R1;
		//terrain[3][5]=R2;
		terrain[1][6]=L0;
		terrain[5][2]=L1;
		terrain[3][8]=L2;
		terrain[5][7]=CS;
 		l0 = 3*4;
        	l1 = 3*4;
        	l2 = 3*4;
		ttl=0;
		csnrj=lmax*3;

		if (observed)
			observer.terrainInit();
	}

	/**
	 * Sets the specified square status, and informs the
	 * observer of the event.
	 *
	 * @param x	The X coordinate of the square to be cut.
	 * @param y	The Y coordinate of the square to be cut.
	 */
	public final void at(int x, int y,Robot robot) {
		if (terrain[x][y] == 0) {
			if (observed)
				observer.robotMoved(x, y);
		}
		if (terrain[x][y] == CS) {
			if (observed)
				observer.robotMoved(x, y);
			if (csnrj>0) {
                                robot.movementAvailable+=6;
                                csnrj-=6;
                        } else {
				robot.movementAvailable+=csnrj;
				csnrj=0;
			}
			return;
		}
		updateTerrain();
	}

	/**
	 * Enables the observer to receive change messages.
	 */
	public void enableObserver() {
		observed = true;
	}

	/**
	 * Disables the observer from receiving change messages.
	 */
	public void disableObserver() {
		observed = false;
	}

	/**
	 * Update terrain's lamp status.
	 */
	public void updateTerrain() {
		l0+=1;	
		if (l0>lmax) l0=lmax;
		l1+=1;	
		if (l1>lmax) l1=lmax;
		l2+=1;	
		if (l2>lmax) l2=lmax;
		ttl++;
		csnrj-=3;
		if (csnrj<0) csnrj=0;
	}
}
